@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#    Copyright (C) 2004-2018 by
+#    Copyright (C) 2004-2019 by
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
@@ -123,6 +123,9 @@ def dijkstra_path(G, source, target, weight='weight'):
 
     Raises
     ------
+    NodeNotFound
+        If `source` is not in `G`.
+
     NetworkXNoPath
        If no path exists between source and target.
 
@@ -197,6 +200,9 @@ def dijkstra_path_length(G, source, target, weight='weight'):
 
     Raises
     ------
+    NodeNotFound
+        If `source` is not in `G`.
+
     NetworkXNoPath
         If no path exists between source and target.
 
@@ -265,6 +271,11 @@ def single_source_dijkstra_path(G, source, cutoff=None, weight='weight'):
     paths : dictionary
        Dictionary of shortest path lengths keyed by target.
 
+    Raises
+    ------
+    NodeNotFound
+        If `source` is not in `G`.
+
     Examples
     --------
     >>> G=nx.path_graph(5)
@@ -325,6 +336,11 @@ def single_source_dijkstra_path_length(G, source, cutoff=None,
     length : dict
         Dict keyed by node to shortest path length from source.
 
+    Raises
+    ------
+    NodeNotFound
+        If `source` is not in `G`.
+
     Examples
     --------
     >>> G = nx.path_graph(5)
@@ -404,6 +420,10 @@ def single_source_dijkstra(G, source, target=None, cutoff=None,
        distance is the distance from source to target and path is a list
        representing the path from source to target.
 
+    Raises
+    ------
+    NodeNotFound
+        If `source` is not in `G`.
 
     Examples
     --------
@@ -512,6 +532,8 @@ def multi_source_dijkstra_path(G, sources, cutoff=None, weight='weight'):
     ------
     ValueError
         If `sources` is empty.
+    NodeNotFound
+        If any of `sources` is not in `G`.
 
     See Also
     --------
@@ -587,6 +609,8 @@ def multi_source_dijkstra_path_length(G, sources, cutoff=None,
     ------
     ValueError
         If `sources` is empty.
+    NodeNotFound
+        If any of `sources` is not in `G`.
 
     See Also
     --------
@@ -689,6 +713,8 @@ def multi_source_dijkstra(G, sources, target=None, cutoff=None,
     ------
     ValueError
         If `sources` is empty.
+    NodeNotFound
+        If any of `sources` is not in `G`.
 
     See Also
     --------
@@ -764,6 +790,11 @@ def _dijkstra_multisource(G, sources, weight, pred=None, paths=None,
         A mapping from node to shortest distance to that node from one
         of the source nodes.
 
+    Raises
+    ------
+    NodeNotFound
+        If any of `sources` is not in `G`.
+
     Notes
     -----
     The optional predecessor and path dictionaries can be accessed by
@@ -782,6 +813,8 @@ def _dijkstra_multisource(G, sources, weight, pred=None, paths=None,
     c = count()
     fringe = []
     for source in sources:
+        if source not in G:
+            raise nx.NodeNotFound("Source {} not in G".format(source))
         seen[source] = 0
         push(fringe, (0, next(c), source))
     while fringe:
@@ -857,6 +890,11 @@ def dijkstra_predecessor_and_distance(G, source, cutoff=None, weight='weight'):
        Warning: If target is specified, the dicts are incomplete as they
        only contain information for the nodes along a path to target.
 
+    Raises
+    ------
+    NodeNotFound
+        If `source` is not in `G`.
+
     Notes
     -----
     Edge weight attributes must be numerical.
@@ -1063,7 +1101,7 @@ def all_pairs_dijkstra_path(G, cutoff=None, weight='weight'):
 
 
 def bellman_ford_predecessor_and_distance(G, source, target=None,
-                                          cutoff=None, weight='weight'):
+                                          weight='weight'):
     """Compute shortest path lengths and predecessors on shortest paths
     in weighted graphs.
 
@@ -1098,11 +1136,12 @@ def bellman_ford_predecessor_and_distance(G, source, target=None,
     pred, dist : dictionaries
        Returns two dictionaries keyed by node to predecessor in the
        path and to the distance from the source respectively.
-       Warning: If target is specified, the dicts are incomplete as they
-       only contain information for the nodes along a path to target.
 
     Raises
     ------
+    NodeNotFound
+        If `source` is not in `G`.
+
     NetworkXUnbounded
        If the (di)graph contains a negative cost (di)cycle, the
        algorithm raises an exception to indicate the presence of the
@@ -1115,15 +1154,15 @@ def bellman_ford_predecessor_and_distance(G, source, target=None,
     >>> G = nx.path_graph(5, create_using = nx.DiGraph())
     >>> pred, dist = nx.bellman_ford_predecessor_and_distance(G, 0)
     >>> sorted(pred.items())
-    [(0, [None]), (1, [0]), (2, [1]), (3, [2]), (4, [3])]
+    [(0, []), (1, [0]), (2, [1]), (3, [2]), (4, [3])]
     >>> sorted(dist.items())
     [(0, 0), (1, 1), (2, 2), (3, 3), (4, 4)]
 
     >>> pred, dist = nx.bellman_ford_predecessor_and_distance(G, 0, 1)
     >>> sorted(pred.items())
-    [(0, [None]), (1, [0])]
+    [(0, []), (1, [0]), (2, [1]), (3, [2]), (4, [3])]
     >>> sorted(dist.items())
-    [(0, 0), (1, 1)]
+    [(0, 0), (1, 1), (2, 2), (3, 3), (4, 4)]
 
     >>> from nose.tools import assert_raises
     >>> G = nx.cycle_graph(5, create_using = nx.DiGraph())
@@ -1143,6 +1182,8 @@ def bellman_ford_predecessor_and_distance(G, source, target=None,
     not containing the source contains a negative cost (di)cycle, it
     will not be detected.
 
+    In NetworkX v2.1 and prior, the source node had predecessor `[None]`.
+    In NetworkX v2.2 this changed to the source node having predecessor `[]`
     """
     if source not in G:
         raise nx.NodeNotFound("Node %s is not found in the graph" % source)
@@ -1151,7 +1192,7 @@ def bellman_ford_predecessor_and_distance(G, source, target=None,
         raise nx.NetworkXUnbounded("Negative cost cycle detected.")
 
     dist = {source: 0}
-    pred = {source: [None]}
+    pred = {source: []}
 
     if len(G) == 1:
         return pred, dist
@@ -1159,26 +1200,30 @@ def bellman_ford_predecessor_and_distance(G, source, target=None,
     weight = _weight_function(G, weight)
 
     dist = _bellman_ford(G, [source], weight, pred=pred, dist=dist,
-                         cutoff=cutoff, target=target)
+                         target=target)
     return (pred, dist)
 
 
 def _bellman_ford(G, source, weight, pred=None, paths=None, dist=None,
-                  cutoff=None, target=None):
-    """Relaxation loop for Bellman–Ford algorithm
+                  target=None):
+    """Relaxation loop for Bellman–Ford algorithm.
+
+    This is an implementation of the SPFA variant.
+    See https://en.wikipedia.org/wiki/Shortest_Path_Faster_Algorithm
 
     Parameters
     ----------
     G : NetworkX graph
 
     source: list
-        List of source nodes
+        List of source nodes. The shortest path from any of the source
+        nodes will be found if multiple sources are provided.
 
     weight : function
-       The weight of an edge is the value returned by the function. The
-       function must accept exactly three positional arguments: the two
-       endpoints of an edge and the dictionary of edge attributes for
-       that edge. The function must return a number.
+        The weight of an edge is the value returned by the function. The
+        function must accept exactly three positional arguments: the two
+        endpoints of an edge and the dictionary of edge attributes for
+        that edge. The function must return a number.
 
     pred: dict of lists, optional (default=None)
         dict to store a list of predecessors keyed by that node
@@ -1193,9 +1238,6 @@ def _bellman_ford(G, source, weight, pred=None, paths=None, dist=None,
         If None, returned dist dict contents default to 0 for every node in the
         source list
 
-    cutoff: integer or float, optional
-        Depth to stop the search. Only paths of length <= cutoff are returned
-
     target: node label, optional
         Ending node for path. Path lengths to other destinations may (and
         probably will) be incorrect.
@@ -1207,15 +1249,21 @@ def _bellman_ford(G, source, weight, pred=None, paths=None, dist=None,
 
     Raises
     ------
+    NodeNotFound
+        If any of `source` is not in `G`.
+
     NetworkXUnbounded
        If the (di)graph contains a negative cost (di)cycle, the
        algorithm raises an exception to indicate the presence of the
        negative cost (di)cycle.  Note: any negative weight edge in an
        undirected graph is a negative cost cycle
     """
+    for s in source:
+        if s not in G:
+            raise nx.NodeNotFound("Source {} not in G".format(s))
 
     if pred is None:
-        pred = {v: [None] for v in source}
+        pred = {v: [] for v in source}
 
     if dist is None:
         dist = {v: 0 for v in source}
@@ -1237,14 +1285,6 @@ def _bellman_ford(G, source, weight, pred=None, paths=None, dist=None,
             for v, e in G_succ[u].items():
                 dist_v = dist_u + weight(v, u, e)
 
-                if cutoff is not None:
-                    if dist_v > cutoff:
-                        continue
-
-                if target is not None:
-                    if dist_v > dist.get(target, inf):
-                        continue
-
                 if dist_v < dist.get(v, inf):
                     if v not in in_q:
                         q.append(v)
@@ -1267,7 +1307,7 @@ def _bellman_ford(G, source, weight, pred=None, paths=None, dist=None,
             path = [dst]
             cur = dst
 
-            while pred[cur][0] is not None:
+            while pred[cur]:
                 cur = pred[cur][0]
                 path.append(cur)
 
@@ -1300,6 +1340,9 @@ def bellman_ford_path(G, source, target, weight='weight'):
 
     Raises
     ------
+    NodeNotFound
+        If `source` is not in `G`.
+
     NetworkXNoPath
        If no path exists between source and target.
 
@@ -1347,6 +1390,9 @@ def bellman_ford_path_length(G, source, target, weight='weight'):
 
     Raises
     ------
+    NodeNotFound
+        If `source` is not in `G`.
+
     NetworkXNoPath
         If no path exists between source and target.
 
@@ -1379,7 +1425,7 @@ def bellman_ford_path_length(G, source, target, weight='weight'):
             "node %s not reachable from %s" % (source, target))
 
 
-def single_source_bellman_ford_path(G, source, cutoff=None, weight='weight'):
+def single_source_bellman_ford_path(G, source, weight='weight'):
     """Compute shortest path between source and all other reachable
     nodes for a weighted graph.
 
@@ -1393,14 +1439,16 @@ def single_source_bellman_ford_path(G, source, cutoff=None, weight='weight'):
     weight: string, optional (default='weight')
        Edge data key corresponding to the edge weight
 
-    cutoff : integer or float, optional
-       Depth to stop the search. Only paths of length <= cutoff are returned.
-
     Returns
     -------
     paths : dictionary
        Dictionary of shortest path lengths keyed by target.
 
+    Raises
+    ------
+    NodeNotFound
+        If `source` is not in `G`.
+
     Examples
     --------
     >>> G=nx.path_graph(5)
@@ -1419,12 +1467,11 @@ def single_source_bellman_ford_path(G, source, cutoff=None, weight='weight'):
 
     """
     (length, path) = single_source_bellman_ford(
-        G, source, cutoff=cutoff, weight=weight)
+        G, source, weight=weight)
     return path
 
 
-def single_source_bellman_ford_path_length(G, source,
-                                           cutoff=None, weight='weight'):
+def single_source_bellman_ford_path_length(G, source, weight='weight'):
     """Compute the shortest path length between source and all other
     reachable nodes for a weighted graph.
 
@@ -1438,14 +1485,16 @@ def single_source_bellman_ford_path_length(G, source,
     weight: string, optional (default='weight')
        Edge data key corresponding to the edge weight.
 
-    cutoff : integer or float, optional
-       Depth to stop the search. Only paths of length <= cutoff are returned.
-
     Returns
     -------
     length : iterator
         (target, shortest path length) iterator
 
+    Raises
+    ------
+    NodeNotFound
+        If `source` is not in `G`.
+
     Examples
     --------
     >>> G = nx.path_graph(5)
@@ -1471,11 +1520,10 @@ def single_source_bellman_ford_path_length(G, source,
 
     """
     weight = _weight_function(G, weight)
-    return _bellman_ford(G, [source], weight, cutoff=cutoff)
+    return _bellman_ford(G, [source], weight)
 
 
-def single_source_bellman_ford(G, source,
-                               target=None, cutoff=None, weight='weight'):
+def single_source_bellman_ford(G, source, target=None, weight='weight'):
     """Compute shortest paths and lengths in a weighted graph G.
 
     Uses Bellman-Ford algorithm for shortest paths.
@@ -1490,9 +1538,6 @@ def single_source_bellman_ford(G, source,
     target : node label, optional
        Ending node for path
 
-    cutoff : integer or float, optional
-       Depth to stop the search. Only paths of length <= cutoff are returned.
-
     Returns
     -------
     distance, path : pair of dictionaries, or numeric and list
@@ -1503,6 +1548,10 @@ def single_source_bellman_ford(G, source,
        distance is the distance from source to target and path is a list
        representing the path from source to target.
 
+    Raises
+    ------
+    NodeNotFound
+        If `source` is not in `G`.
 
     Examples
     --------
@@ -1542,8 +1591,7 @@ def single_source_bellman_ford(G, source,
     weight = _weight_function(G, weight)
 
     paths = {source: [source]}  # dictionary of paths
-    dist = _bellman_ford(G, [source], weight, paths=paths, cutoff=cutoff,
-                         target=target)
+    dist = _bellman_ford(G, [source], weight, paths=paths, target=target)
     if target is None:
         return (dist, paths)
     try:
@@ -1553,7 +1601,7 @@ def single_source_bellman_ford(G, source,
         raise nx.NetworkXNoPath(msg)
 
 
-def all_pairs_bellman_ford_path_length(G, cutoff=None, weight='weight'):
+def all_pairs_bellman_ford_path_length(G, weight='weight'):
     """ Compute shortest path lengths between all nodes in a weighted graph.
 
     Parameters
@@ -1563,9 +1611,6 @@ def all_pairs_bellman_ford_path_length(G, cutoff=None, weight='weight'):
     weight: string, optional (default='weight')
        Edge data key corresponding to the edge weight
 
-    cutoff : integer or float, optional
-       Depth to stop the search. Only paths of length <= cutoff are returned.
-
     Returns
     -------
     distance : iterator
@@ -1597,10 +1642,10 @@ def all_pairs_bellman_ford_path_length(G, cutoff=None, weight='weight'):
     """
     length = single_source_bellman_ford_path_length
     for n in G:
-        yield (n, dict(length(G, n, cutoff=cutoff, weight=weight)))
+        yield (n, dict(length(G, n, weight=weight)))
 
 
-def all_pairs_bellman_ford_path(G, cutoff=None, weight='weight'):
+def all_pairs_bellman_ford_path(G, weight='weight'):
     """ Compute shortest paths between all nodes in a weighted graph.
 
     Parameters
@@ -1610,9 +1655,6 @@ def all_pairs_bellman_ford_path(G, cutoff=None, weight='weight'):
     weight: string, optional (default='weight')
        Edge data key corresponding to the edge weight
 
-    cutoff : integer or float, optional
-       Depth to stop the search. Only paths of length <= cutoff are returned.
-
     Returns
     -------
     distance : dictionary
@@ -1638,7 +1680,7 @@ def all_pairs_bellman_ford_path(G, cutoff=None, weight='weight'):
     path = single_source_bellman_ford_path
     # TODO This can be trivially parallelized.
     for n in G:
-        yield (n, path(G, n, cutoff=cutoff, weight=weight))
+        yield (n, path(G, n, weight=weight))
 
 
 def goldberg_radzik(G, source, weight='weight'):
@@ -1679,6 +1721,9 @@ def goldberg_radzik(G, source, weight='weight'):
 
     Raises
     ------
+    NodeNotFound
+        If `source` is not in `G`.
+
     NetworkXUnbounded
        If the (di)graph contains a negative cost (di)cycle, the
        algorithm raises an exception to indicate the presence of the
@@ -1820,7 +1865,7 @@ def goldberg_radzik(G, source, weight='weight'):
 
 
 def negative_edge_cycle(G, weight='weight'):
-    """Return True if there exists a negative edge cycle anywhere in G.
+    """Returns True if there exists a negative edge cycle anywhere in G.
 
     Parameters
     ----------
@@ -1877,7 +1922,7 @@ def negative_edge_cycle(G, weight='weight'):
 
 
 def bidirectional_dijkstra(G, source, target, weight='weight'):
-    """Dijkstra's algorithm for shortest paths using bidirectional search.
+    r"""Dijkstra's algorithm for shortest paths using bidirectional search.
 
     Parameters
     ----------
@@ -1910,6 +1955,9 @@ def bidirectional_dijkstra(G, source, target, weight='weight'):
 
     Raises
     ------
+    NodeNotFound
+        If either `source` or `target` is not in `G`.
+
     NetworkXNoPath
         If no path exists between source and target.
 
@@ -2096,7 +2144,7 @@ def johnson(G, weight='weight'):
         raise nx.NetworkXError('Graph is not weighted.')
 
     dist = {v: 0 for v in G}
-    pred = {v: [None] for v in G}
+    pred = {v: [] for v in G}
     weight = _weight_function(G, weight)
 
     # Calculate distance of shortest paths
