@@ -19,7 +19,7 @@ This implementation is based on:
 #    doi={10.1007/BF02579168},
 #    title={Efficient algorithms for finding minimum spanning trees in
 #        undirected and directed graphs},
-#    url={http://dx.doi.org/10.1007/BF02579168},
+#    url={https://doi.org/10.1007/BF02579168},
 #    publisher={Springer-Verlag},
 #    keywords={68 B 15; 68 C 05},
 #    author={Gabow, Harold N. and Galil, Zvi and Spencer, Thomas and Tarjan,
@@ -36,6 +36,7 @@ import random
 from operator import itemgetter
 
 import networkx as nx
+from networkx.utils import py_random_state
 
 from .recognition import *
 
@@ -56,16 +57,20 @@ STYLES = {
 
 INF = float('inf')
 
+
+@py_random_state(1)
 def random_string(L=15, seed=None):
-    random.seed(seed)
-    return ''.join([random.choice(string.ascii_letters) for n in range(L)])
+    return ''.join([seed.choice(string.ascii_letters) for n in range(L)])
+
 
 def _min_weight(weight):
     return -weight
 
+
 def _max_weight(weight):
     return weight
 
+
 def branching_weight(G, attr='weight', default=1):
     """
     Returns the total weight of a branching.
@@ -73,7 +78,9 @@ def branching_weight(G, attr='weight', default=1):
     """
     return sum(edge[2].get(attr, default) for edge in G.edges(data=True))
 
-def greedy_branching(G, attr='weight', default=1, kind='max'):
+
+@py_random_state(4)
+def greedy_branching(G, attr='weight', default=1, kind='max', seed=None):
     """
     Returns a branching obtained through a greedy algorithm.
 
@@ -96,6 +103,9 @@ def greedy_branching(G, attr='weight', default=1, kind='max'):
         `default` specifies what value it should take.
     kind : str
         The type of optimum to search for: 'min' or 'max' greedy branching.
+    seed : integer, random_state, or None (default)
+        Indicator of random number generation state.
+        See :ref:`Randomness<randomness>`.
 
     Returns
     -------
@@ -113,7 +123,7 @@ def greedy_branching(G, attr='weight', default=1, kind='max'):
 
     if attr is None:
         # Generate a random string the graph probably won't have.
-        attr = random_string()
+        attr = random_string(seed=seed)
 
     edges = [(u, v, data.get(attr, default))
              for (u, v, data) in G.edges(data=True)]
@@ -149,6 +159,7 @@ def greedy_branching(G, attr='weight', default=1, kind='max'):
 
     return B
 
+
 class MultiDiGraph_EdgeKey(nx.MultiDiGraph):
     """
     MultiDiGraph which assigns unique keys to every edge.
@@ -165,9 +176,10 @@ class MultiDiGraph_EdgeKey(nx.MultiDiGraph):
     of edges. We must reliably track edges across graph mutations.
 
     """
-    def __init__(self, data=None, **attr):
+
+    def __init__(self, incoming_graph_data=None, **attr):
         cls = super(MultiDiGraph_EdgeKey, self)
-        cls.__init__(data=data, **attr)
+        cls.__init__(incoming_graph_data=incoming_graph_data, **attr)
 
         self._cls = cls
         self.edge_index = {}
@@ -188,30 +200,27 @@ class MultiDiGraph_EdgeKey(nx.MultiDiGraph):
         for n in nbunch:
             self.remove_node(n)
 
-    def fresh_copy(self):
-        # Needed to make .copy() work
-        return MultiDiGraph_EdgeKey()
-
-    def add_edge(self, u, v, key, **attr):
+    def add_edge(self, u_for_edge, v_for_edge, key_for_edge, **attr):
         """
         Key is now required.
 
         """
+        u, v, key = u_for_edge, v_for_edge, key_for_edge
         if key in self.edge_index:
             uu, vv, _ = self.edge_index[key]
             if (u != uu) or (v != vv):
                 raise Exception("Key {0!r} is already in use.".format(key))
 
-        self._cls.add_edge(u, v, key=key, **attr)
+        self._cls.add_edge(u, v, key, **attr)
         self.edge_index[key] = (u, v, self.succ[u][v][key])
 
-    def add_edges_from(self, ebunch, **attr):
-        for u, v, k, d in ebunch:
+    def add_edges_from(self, ebunch_to_add, **attr):
+        for u, v, k, d in ebunch_to_add:
             self.add_edge(u, v, k, **d)
 
     def remove_edge_with_key(self, key):
         try:
-            u, v, _  = self.edge_index[key]
+            u, v, _ = self.edge_index[key]
         except KeyError:
             raise KeyError('Invalid edge key {0!r}'.format(key))
         else:
@@ -244,11 +253,13 @@ def get_path(G, u, v):
     edges = [first_key(i, vv) for i, vv in enumerate(nodes[1:])]
     return nodes, edges
 
+
 class Edmonds(object):
     """
     Edmonds algorithm for finding optimal branchings and spanning arborescences.
 
     """
+
     def __init__(self, G, seed=None):
         self.G_original = G
 
@@ -262,7 +273,8 @@ class Edmonds(object):
         # sure that our node names do not conflict with the real node names.
         self.template = random_string(seed=seed) + '_{0}'
 
-    def _init(self, attr, default, kind, style):
+
+    def _init(self, attr, default, kind, style, preserve_attrs, seed):
         if kind not in KINDS:
             raise nx.NetworkXException("Unknown value for `kind`.")
 
@@ -280,15 +292,26 @@ class Edmonds(object):
 
         if attr is None:
             # Generate a random attr the graph probably won't have.
-            attr = random_string()
+            attr = random_string(seed=seed)
 
         # This is the actual attribute used by the algorithm.
         self._attr = attr
 
+        # This attribute is used to store whether a particular edge is still
+        # a candidate. We generate a random attr to remove clashes with
+        # preserved edges
+        self.candidate_attr = 'candidate_' + random_string(seed=seed)
+
         # The object we manipulate at each step is a multidigraph.
         self.G = G = MultiDiGraph_EdgeKey()
         for key, (u, v, data) in enumerate(self.G_original.edges(data=True)):
             d = {attr: trans(data.get(attr, default))}
+
+            if preserve_attrs:
+                for (d_k, d_v) in data.items():
+                    if d_k != attr:
+                        d[d_k] = d_v
+
             G.add_edge(u, v, key, **d)
 
         self.level = 0
@@ -316,7 +339,8 @@ class Edmonds(object):
         # in circuit G^0 (depsite their weights being different).
         self.minedge_circuit = []
 
-    def find_optimum(self, attr='weight', default=1, kind='max', style='branching'):
+    def find_optimum(self, attr='weight', default=1, kind='max',
+                     style='branching', preserve_attrs=False, seed=None):
         """
         Returns a branching from G.
 
@@ -335,6 +359,12 @@ class Edmonds(object):
             branching is also an arborescence, then the branching is an
             optimal spanning arborescences. A given graph G need not have
             an optimal spanning arborescence.
+        preserve_attrs : bool
+            If True, preserve the other edge attributes of the original
+            graph (that are not the one passed to `attr`)
+        seed : integer, random_state, or None (default)
+            Indicator of random number generation state.
+            See :ref:`Randomness<randomness>`.
 
         Returns
         -------
@@ -342,7 +372,7 @@ class Edmonds(object):
             The branching.
 
         """
-        self._init(attr, default, kind, style)
+        self._init(attr, default, kind, style, preserve_attrs, seed)
         uf = self.uf
 
         # This enormous while loop could use some refactoring...
@@ -371,15 +401,15 @@ class Edmonds(object):
         while True:
             # (I1): Choose a node v in G^i not in D^i.
             try:
-               v = next(nodes)
+                v = next(nodes)
             except StopIteration:
                 # If there are no more new nodes to consider, then we *should*
                 # meet the break condition (b) from the paper:
                 #   (b) every node of G^i is in D^i and E^i is a branching
                 # Construction guarantees that it's a branching.
-                assert( len(G) == len(B) )
+                assert(len(G) == len(B))
                 if len(B):
-                    assert( is_branching(B) )
+                    assert(is_branching(B))
 
                 if self.store:
                     self.graphs.append(G.copy())
@@ -433,8 +463,8 @@ class Edmonds(object):
                 #print("Edge is acceptable: {0}".format(acceptable))
                 if acceptable:
                     dd = {attr: weight}
-                    B.add_edge(u, v, key=edge[2], **dd)
-                    G[u][v][edge[2]]['candidate'] = True
+                    B.add_edge(u, v, edge[2], **dd)
+                    G[u][v][edge[2]][self.candidate_attr] = True
                     uf.union(u, v)
                     if Q_edges is not None:
                         #print("Edge introduced a simple cycle:")
@@ -453,7 +483,7 @@ class Edmonds(object):
                             u, v, data = B.edge_index[edge_key]
                             w = data[attr]
                             Q_incoming_weight[v] = w
-                            if  w < minweight:
+                            if w < minweight:
                                 minweight = w
                                 minedge = edge_key
 
@@ -500,19 +530,17 @@ class Edmonds(object):
 
                         for u, v, key, data in new_edges:
                             G.add_edge(u, v, key, **data)
-                            if 'candidate' in data:
-                                del data['candidate']
+                            if self.candidate_attr in data:
+                                del data[self.candidate_attr]
                                 B.add_edge(u, v, key, **data)
                                 uf.union(u, v)
 
                         nodes = iter(list(G.nodes()))
                         self.level += 1
 
-
-
         # (I3) Branch construction.
-        #print(self.level)
-        H = self.G_original.fresh_copy()
+        # print(self.level)
+        H = self.G_original.__class__()
 
         def is_root(G, u, edgekeys):
             """
@@ -548,7 +576,7 @@ class Edmonds(object):
             # The circuit at level i that was merged as a node the graph
             # at level i+1.
             circuit = self.circuits[self.level]
-            #print
+            # print
             #print(merged_node, self.level, circuit)
             #print("before", edges)
             # Note, we ask if it is a root in the full graph, not the branching.
@@ -574,7 +602,7 @@ class Edmonds(object):
                 #print("circuit is: ", circuit)
                 # The branching at level i
                 G = self.graphs[self.level]
-                #print(G.edge_index)
+                # print(G.edge_index)
                 target = G.edge_index[edgekey][1]
                 for edgekey in circuit:
                     u, v, data = G.edge_index[edgekey]
@@ -592,38 +620,56 @@ class Edmonds(object):
         for edgekey in edges:
             u, v, d = self.graphs[0].edge_index[edgekey]
             dd = {self.attr: self.trans(d[self.attr])}
-            # TODO: make this preserve the key. In fact, make this use the
-            # same edge attributes as the original graph.
+
+            # Optionally, preserve the other edge attributes of the original
+            # graph
+            if preserve_attrs:
+                for (key, value) in d.items():
+                    if key not in [self.attr, self.candidate_attr]:
+                        dd[key] = value
+
+            # TODO: make this preserve the key.
             H.add_edge(u, v, **dd)
 
         return H
 
-def maximum_branching(G, attr='weight', default=1):
+
+def maximum_branching(G, attr='weight', default=1, preserve_attrs=False):
     ed = Edmonds(G)
-    B = ed.find_optimum(attr, default, kind='max', style='branching')
+    B = ed.find_optimum(attr, default, kind='max', style='branching',
+                        preserve_attrs=preserve_attrs)
     return B
 
-def minimum_branching(G, attr='weight', default=1):
+
+def minimum_branching(G, attr='weight', default=1, preserve_attrs=False):
     ed = Edmonds(G)
-    B = ed.find_optimum(attr, default, kind='min', style='branching')
+    B = ed.find_optimum(attr, default, kind='min', style='branching',
+                        preserve_attrs=preserve_attrs)
     return B
 
-def maximum_spanning_arborescence(G, attr='weight', default=1):
+
+def maximum_spanning_arborescence(G, attr='weight', default=1,
+                                  preserve_attrs=False):
     ed = Edmonds(G)
-    B = ed.find_optimum(attr, default, kind='max', style='arborescence')
+    B = ed.find_optimum(attr, default, kind='max', style='arborescence',
+                        preserve_attrs=preserve_attrs)
     if not is_arborescence(B):
         msg = 'No maximum spanning arborescence in G.'
         raise nx.exception.NetworkXException(msg)
     return B
 
-def minimum_spanning_arborescence(G, attr='weight', default=1):
+
+def minimum_spanning_arborescence(G, attr='weight', default=1,
+                                  preserve_attrs=False):
     ed = Edmonds(G)
-    B = ed.find_optimum(attr, default, kind='min', style='arborescence')
+    B = ed.find_optimum(attr, default, kind='min', style='arborescence',
+                        preserve_attrs=preserve_attrs)
     if not is_arborescence(B):
         msg = 'No minimum spanning arborescence in G.'
         raise nx.exception.NetworkXException(msg)
     return B
 
+
 docstring_branching = """
 Returns a {kind} {style} from G.
 
@@ -636,6 +682,9 @@ attr : str
 default : float
     The value of the edge attribute used if an edge does not have
     the attribute `attr`.
+preserve_attrs : bool
+    If True, preserve the other attributes of the original graph (that are not
+    passed to `attr`)
 
 Returns
 -------
