@@ -7,18 +7,18 @@ Do not add this file to the repository.
 
 import datetime
 
-version = '2.2'
-date = 'Wed Sep 26 12:42:48 2018'
+version = '2.3'
+date = 'Wed Jun 26 12:44:59 2019'
 
 # Was NetworkX built from a development version? If so, remember that the major
 # and minor versions reference the "target" (rather than "current") release.
 dev = False
 
 # Format: (name, major, min, revision)
-version_info = ('networkx', '2', '2', None)
+version_info = ('networkx', '2', '3', None)
 
 # Format: a 'datetime.datetime' instance
-date_info = datetime.datetime(2018, 9, 26, 12, 42, 48)
+date_info = datetime.datetime(2019, 6, 26, 12, 44, 59)
 
 # Format: (vcs, vcs_tuple)
 vcs_info = (None, (None, None))
